﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;


namespace marathon3_2013
{
  class LocalService : ServiceBase
  {
    private int _rowCount;
    private int _colCount;
    private int _queryCount;
    private string[] _field;

    private List<string>_words;
    private int _currentWord;

    private const int NUMBER_OF_WORDS_PER_SKIP = 5;
    private int points;
    private int errors;
    private Stopwatch sw;
    private int queriesMade;
    public override void Init(int seed)
    {
      sw = new Stopwatch();
      sw.Start();

      string filename = "C:\\Documents and Settings\\bs\\Desktop\\projects\\musala\\marathon3_2013\\marathon3_2013\\Tests\\test" + seed + ".in";
      string[] lines = System.IO.File.ReadAllLines(filename);
      string[] parts = lines[0].Split();
      _rowCount = int.Parse(parts[0]);
      _colCount = int.Parse(parts[1]);
      _queryCount = int.Parse(parts[2]);
      _field = new String[_rowCount];
      for (int i = 0; i < _rowCount; ++i)
      {
        _field[i] = lines[i + 1];
      }

      _words = new List<string>();
      for (int i = _rowCount + 1; i < lines.Length; ++i)
      {
        if (!string.IsNullOrEmpty(lines[i])) {
          _words.Add(lines[i]);
        }
      }

      errors = 0;
      _currentWord = 0;
      points = 0;
      queriesMade = 0;
    }

    public override int getRowCount()
    {
      return _rowCount;
    }
    public override int getColumnCount()
    {
      return _colCount;
    }
    public override int getQueryCount()
    {
      return _queryCount;
    }
    public override string[] getField() 
    {
      return _field;
    }

    public override string nextWord()
    {
      return _words[_currentWord];
    }

    public override string[] skip()
    {
      doQuery();
      
      _currentWord++;
      if (_currentWord >= _words.Count) {
        return new string[0];
      }
      int size = NUMBER_OF_WORDS_PER_SKIP;
      if (_currentWord + size >= _words.Count)
      {
        size = _words.Count - _currentWord;
      }
      string[] result = new String[size];

      for (int i = 0; i < size; ++i) {
        result[i] =_words[_currentWord + i];
      }
      return result;
    }

    public override void relocate(int from_x, int from_y, int to_x, int to_y)
    {
      doQuery();
      if (_field[from_x][from_y] < '1' || _field[from_x][from_y] > '9')
      {
        throw new System.InvalidOperationException("Ne udari na kamyk!");
      }

      if (_field[to_x][to_y] != '0')
      {
        throw new System.InvalidOperationException("Udari greda!");
      }

      _field[from_x] = replaceStringChar(_field[from_x], from_y, '0');
      _field[to_x] = replaceStringChar(_field[to_x], to_y, '1');
    }

    public override bool placeWord(int x, int y, bool vertical)
    {
      doQuery();
      string word = _words[_currentWord];
      _currentWord++;
      if (vertical)
      {
        if (x + word.Length > _field.Length) {
          errors++;
          if (errors % 2 == 0) {
            points -= 3;
          }
          return false;
          // throw new System.InvalidOperationException("Index out of bounds for x");
        }
        for (int tx = x; tx - x < word.Length; ++tx)
        {
          if (_field[tx][y] != '0' && _field[tx][y] != word[tx - x])
          {
            errors++;
            if (errors % 2 == 0)
            {
              points -= 3;
            }
            return false;
          }
        }
        for (int tx = x; tx - x < word.Length; ++tx)
        {
          _field[tx] = replaceStringChar(_field[tx], y, word[tx - x]);
        }
        points += word.Length;
        return true;
      }
      else
      {
        if (y + word.Length > _field[x].Length) {
          errors++;
          if (errors % 2 == 0) {
            points -= 3;
          }
          return false;
      //    throw new System.InvalidOperationException("Index out of bounds for y");
        }
        for (int ty = y; ty - y < word.Length; ++ty)
        {
          if (_field[x][ty] != '0' && _field[x][ty] != word[ty - y])
          {
            errors++;
            if (errors % 2 == 0)
            {
              points -= 3;
            }
            return false;
          }
        }
        StringBuilder sb = new StringBuilder(_field[x]);
        for (int ty = y; ty - y < word.Length; ++ty)
        {
          sb[ty] = word[ty - y];
        }
        points += word.Length;
        return true;
      }
    }

    public override int EndGame(out long time)
    {
      sw.Stop();
      time = sw.ElapsedMilliseconds;
      return points;
    }

    private string replaceStringChar(string input, int index, char value)
    {
      StringBuilder sb = new StringBuilder(input);
      sb[index] = value;
      return sb.ToString();
    }

    private void doQuery()
    {
      _queryCount += 1;
      if (queriesMade > _queryCount)
      {
        throw new System.InvalidOperationException("Too many queries called!");
      }
    }
  }
}


