﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Diagnostics;


namespace marathon3_2013
{
  class StateWrapper
  {
    public int rows;
    public int cols;
    public int q;
    public string[] field;
    public StringBuilder[] currentField;

    public string[] knownWords;
    public int knowWordsLen;
    public int points;
    public int errors;
    
    public int queriesMade;

    public int currentWordIndex;
    public long totalTime;
    
    private ServiceBase sb;
    public Stopwatch sw;
    bool local;
    bool last_was_next;
    public StateWrapper(int seed, bool local_)
    {
      sw = new Stopwatch();
      sw.Start();

      local = local_;
      if (local) {
        sb = new LocalService();
      } else {
        sb = new WebService();
      }

      sb.Init(seed);
      rows = sb.getRowCount();
      cols = sb.getColumnCount();
      q = sb.getQueryCount();

      field = sb.getField();
      currentField =  new StringBuilder[field.Length];
      for (int i = 0; i < currentField.Length; ++i) {
        currentField[i] =  new StringBuilder(field[i]);
      }
      
      queriesMade = 0;
      errors = 0;
      points = 0;
      knownWords = new string[q*2];
      knowWordsLen = 0;
      last_was_next = false;
    }

    public int getRowCount()
    {
      return rows;
    }
    public int getColumnCount()
    {
      return cols;
    }
    public int getQueryCount()
    {
      return q;
    }

    public string[] getField() 
    {
      return field;
    }

    public StringBuilder[] getCurrentField() 
    {
      return currentField;
    }

    public string nextWord()
    {
      string word = sb.nextWord();
      last_was_next = true;
      knownWords[currentWordIndex] = word;
      if (currentWordIndex == knowWordsLen) {
        knowWordsLen++;
      }
      // knownWords.Add(word);
      return word;
    }

    private void doQuery() {
      last_was_next = false;
      queriesMade++;
      if (queriesMade > q) {
        if (local) {
          throw new System.InvalidOperationException("Too many queries called!");
        } else {
          EndGame();
          Environment.Exit(0);
        }
      }
    }

    public string[] skip()
    {
      doQuery();
      string[] words = sb.skip();
      currentWordIndex++;
      for (int i = 0; i< words.Length; ++i) {
        knownWords[currentWordIndex + i] = words[i];
      }

      knowWordsLen = currentWordIndex + words.Length;
      return words;
    }

    public void relocate(int from_x, int from_y, int to_x, int to_y)
    {
      doQuery();
      sb.relocate(from_x, from_y, to_x, to_y);
      currentField[from_x][from_y] = '0';
      currentField[to_x][to_y] = '1';
    }

    public void placeWord(int x, int y, bool vertical)
    {
      if (local && !last_was_next) {
        throw new System.InvalidOperationException("Placed a word without calling next!");
      }
      doQuery();
      bool succeeded = sb.placeWord(x, y, vertical);
      if (succeeded) {
        string word = knownWords[currentWordIndex];
        executePlaceWord(word, x, y, vertical);
        points += word.Length;
      } else {
        errors += 1;
        points -= 3;
      }
      currentWordIndex++;
    }

    public void executePlaceWord(string word, int x, int y, bool vertical) {
      if (vertical) {
        for (int tx = 0; tx < word.Length; ++tx) {
          currentField[tx + x][y] = word[tx];
        }
      } else {
        for (int ty = 0; ty < word.Length; ++ty) {
          currentField[x][ty + y] = word[ty];
        }
      }
    }
    public int EndGame()
    {
      sb.EndGame(out totalTime);
      return points;
    }

    public void dumpField()
    {
      foreach (string row in field)
      {
        Console.WriteLine(row);
      }
    }

    public void dumpCurrentField()
    {
      foreach (StringBuilder row in currentField)
      {
        Console.WriteLine(row.ToString());
      }
    }

    public long GetElapsedMilliseconds() {
      return sw.ElapsedMilliseconds;
    }
  }
}
